/* knocker version 0.2.0
 * Release date: 16 July 2001
 *
 * Project homepage: http://knocker.sourceforge.net
 *
 * Copyright 2001 Gabriele Giorgetti <g.gabriele@europe.com>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "knocker_core.h"

#ifndef FALSE
#  define FALSE 0
#endif
#ifndef TRUE
#  define TRUE 1
#endif


/* COMMAND LINE ARGUMENTS DEFINITION  */
/* ********************************** */
  /* host to scan */
#define HOST_SHORT_OPT "-H"
#define HOST_LONG_OPT  "--host"

  /* port number to start with */
#define START_PORT_SHORT_OPT "-SP"
#define START_PORT_LONG_OPT  "--start-port"

  /* port number to scan to */
#define END_PORT_SHORT_OPT "-EP"
#define END_PORT_LONG_OPT  "--end-port"

#define HELP_SHORT_OPT    "-h"
#define HELP_LONG_OPT     "--help"

#define VERSION_SHORT_OPT "-v"
#define VERSION_LONG_OPT  "--version"
/* ********************************** */


#define RELEASE_DATE "16 July 2001"
#define AUTHOR_EMAIL "g.gabriele@europe.com"
#define KNOCKER_HOMEPAGE "http://knocker.sourceforge.net"

char *HOSTNAME_STRING;
char *HOSTIP_STRING;
char *SERVICENAME_STRING;

int START_PORT_NUMBER = 0;
int END_PORT_NUMBER = 0;


int set_hostname_string (const char *hostname);
void unset_hostname_string (void);
int set_hostip_string (int lenght);
void unset_hostip_string (void);

int parse_args (int argc, char *argv[]);
int check_port_number (int port);
void print_version_info (void);
void print_help_info ( void );
void quit (int state);


int set_hostname_string (const char *hostname)
{
  HOSTNAME_STRING = malloc (strlen (hostname) + 1);

  strcpy (HOSTNAME_STRING, hostname);

  return 0;
}


void unset_hostname_string (void)
{
  if (HOSTNAME_STRING != NULL)
    free (HOSTNAME_STRING);

  HOSTNAME_STRING = NULL;
}

int set_hostip_string (int lenght)
{
  HOSTIP_STRING = malloc (lenght +1);
  
  return 0;
}

void unset_hostip_string (void)
{
  if (HOSTIP_STRING != NULL)
      free (HOSTIP_STRING);
      
  HOSTIP_STRING = NULL;
}
        

int set_servicename_string (int lenght)
{
  SERVICENAME_STRING = malloc (lenght + 1);

  return 0;
}

void unset_servicename_string (void)
{
  if (SERVICENAME_STRING != NULL)
    free (SERVICENAME_STRING);

  SERVICENAME_STRING = NULL;
}

int is_port_number_valid ( int port )
{
  if ( port < 0 )
    return FALSE;
  if ( port > KNOCKER_MAX_PORT_NUMBER)
    return FALSE;

  return TRUE;
}

void quit (int state)
{

  unset_hostname_string ();
  unset_hostip_string();
  unset_servicename_string();

  exit (state);
}


void print_version_info (void)
{
  fprintf (stdout, "%s %s (%s)\n", PACKAGE, VERSION, RELEASE_DATE);
  fprintf (stdout, "\n");
  fprintf (stdout, "Written by Gabriele Giorgetti <%s>\n", AUTHOR_EMAIL);
  fprintf (stdout, "Copyright (C) 2001 Gabriele Giorgetti\n");
  fprintf (stdout, "\n");
  fprintf (stdout, "This software is released under the GNU GPL\n");
  fprintf (stdout, "\n");
  fprintf (stdout, "%s can be found at %s\n", PACKAGE, KNOCKER_HOMEPAGE);
}


void print_help_info (void)
{
  fprintf (stdout, "%s, the net portscanner. Version %s (%s)\n", PACKAGE, VERSION, RELEASE_DATE);
  fprintf (stdout, "\n");
  fprintf (stdout, "Usage: %s %s <HOST> \n", PACKAGE,  HOST_LONG_OPT);
  fprintf (stdout, "   or: %s %s <HOST> %s <PORT> %s <PORT> \n", PACKAGE, HOST_LONG_OPT, START_PORT_LONG_OPT, END_PORT_LONG_OPT );
  fprintf (stdout, "\n");
  fprintf (stdout, "Example: %s %s 192.168.0.1 %s 1 %s 1024\n", PACKAGE, HOST_SHORT_OPT, START_PORT_SHORT_OPT, END_PORT_SHORT_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Required options:\n");
  fprintf (stdout, "      %s,  %s             host name or numeric Internet address\n", HOST_SHORT_OPT, HOST_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Common options (if SP is specified you must also give EP):\n");
  fprintf (stdout, "      %s, %s       port number to begin the scan from\n", START_PORT_SHORT_OPT, START_PORT_LONG_OPT);
  fprintf (stdout, "      %s, %s         port number to end the scan at\n", END_PORT_SHORT_OPT, END_PORT_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "      %s,  %s             display this help and exit\n", HELP_SHORT_OPT, HELP_LONG_OPT);
  fprintf (stdout, "      %s,  %s          output version information and exit\n", VERSION_SHORT_OPT, VERSION_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "SEE THE MAN PAGE FOR MORE DESCRIPTIONS, AND EXAMPLES\n");
  fprintf (stdout, "Report bugs to <%s>.\n", AUTHOR_EMAIL);
}



int parse_args (int argc, char *argv[])
{
  int i;

  if (argv[1] == NULL)
    {
      fprintf (stderr, "%s: no arguments given\n", PACKAGE);
      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE,
	       HELP_LONG_OPT);
      quit (1);
    }

  for (i = 1; i <= argc - 1; i++)
    {
      if((!strcmp (argv[i], VERSION_SHORT_OPT))
         || (!strcmp (argv[i], VERSION_LONG_OPT)))
	{
	  print_version_info ();
	  quit (0);
	}
      else if((!strcmp (argv[i], HELP_SHORT_OPT))
         || (!strcmp (argv[i], HELP_LONG_OPT)))
        {
	  print_help_info();
	  quit(0);
	}
      else if ((!strcmp (argv[i], HOST_SHORT_OPT))
	       || (!strcmp (argv[i], HOST_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr, "%s: error, no host to scan given\n", PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE,
		       HELP_LONG_OPT);
	      quit (1);
	    }

	  set_hostname_string (argv[i]);
	}

      else if ((!strcmp (argv[i], START_PORT_SHORT_OPT))
	       || (!strcmp (argv[i], START_PORT_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr,
		       "%s: error, no port number, where to start the scan from, given\n",
		       PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE,
		       HELP_LONG_OPT);
	      quit (1);
	    }

	  START_PORT_NUMBER = atoi (argv[i]);

	  if (is_port_number_valid(START_PORT_NUMBER) != TRUE)
	    {
	      fprintf (stderr,"%s: error, invalid start port number `%d' given\n", PACKAGE, START_PORT_NUMBER);
              quit(1);
	    }
	}
      else if ((!strcmp (argv[i], END_PORT_SHORT_OPT))
	       || (!strcmp (argv[i], END_PORT_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr,
		       "%s: error, no port number, where to scan to, given\n",
		       PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE,
		       HELP_LONG_OPT);
	      quit (1);
	    }

	  END_PORT_NUMBER = atoi (argv[i]);

	  if (is_port_number_valid(END_PORT_NUMBER) != TRUE)
	    {
	      fprintf (stderr,"%s: error, invalid end port number `%d' given\n", PACKAGE, END_PORT_NUMBER);
              quit(1);
	    }
	}
       else
        {
           fprintf (stderr, "%s: invalid arguments given\n", PACKAGE);
           fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE,
	       HELP_LONG_OPT);
           quit (1);
	}


    }				/* end of for loop */

  return 0;
}




int main (int argc, char *argv[])
{
  int open_ports    = 0;
  int scanned_ports = 0;

  parse_args (argc, argv);
  
  set_servicename_string (20);
  set_hostip_string(40);
  
  fprintf (stdout, "\n");

  if ( (START_PORT_NUMBER == 0 ) && ( END_PORT_NUMBER == 0 ) )
    {
       /* default portscan */
       int port;
       
       START_PORT_NUMBER = 1;
       END_PORT_NUMBER = KNOCKER_DEFAULT_PORT_RANGE;
       
       fprintf(stdout, "Resolving host: %s ---", HOSTNAME_STRING);
       
       if (knocker_get_ip_by_hostname (HOSTIP_STRING, HOSTNAME_STRING) != 0)
         {
           fprintf(stdout, "| couldn't resolve hostname.\n");
           fprintf(stdout, "Aborting.\n\n");
           quit(1);
         }
         
       fprintf(stdout, "---> %s\n", HOSTIP_STRING);
       
       unset_hostip_string();   
                 
       fprintf(stdout, "Scanning host: %s from port: %d to port: %d\n\n", HOSTNAME_STRING, START_PORT_NUMBER,  END_PORT_NUMBER);

       for (port = START_PORT_NUMBER; port < END_PORT_NUMBER; port++)
          {
            if (knocker_portscan_by_hostname (HOSTNAME_STRING, port) == PORT_IS_OPEN)
	      {
	        open_ports++;
	        knocker_get_service_by_port (SERVICENAME_STRING, port);
                fprintf(stdout, "%d/tcp, %s, --> open\n", port, SERVICENAME_STRING);
	      }

	  }	  
       scanned_ports = END_PORT_NUMBER - START_PORT_NUMBER;
    }

  else if ( START_PORT_NUMBER > END_PORT_NUMBER )
    {
       /* reverse scan */
       int port;

       fprintf(stdout, "Scanning host: %s from port: %d to port: %d\n\n", HOSTNAME_STRING, START_PORT_NUMBER, END_PORT_NUMBER);

       for (port = START_PORT_NUMBER; port > END_PORT_NUMBER; port--)
          {
            if (knocker_portscan_by_hostname (HOSTNAME_STRING, port) == PORT_IS_OPEN)
	      {
	        open_ports++;
	        knocker_get_service_by_port (SERVICENAME_STRING, port);
                fprintf(stdout, "%d/tcp, %s --> open\n", port, SERVICENAME_STRING);
	      }

	  }
       scanned_ports = START_PORT_NUMBER - END_PORT_NUMBER ;
    }
  else
    {
       /* scan from START_PORT_NUMBER to END_PORT_NUMBER */
       int port;

       fprintf(stdout, "Scanning host: %s from port: %d to port: %d\n\n", HOSTNAME_STRING, START_PORT_NUMBER, END_PORT_NUMBER);

       for (port = START_PORT_NUMBER; port < END_PORT_NUMBER; port++)
          {
            if (knocker_portscan_by_hostname (HOSTNAME_STRING, port) == PORT_IS_OPEN)
	      {
	        open_ports++;
	        knocker_get_service_by_port (SERVICENAME_STRING, port);
                fprintf(stdout, "%d/tcp, %s --> open\n", port, SERVICENAME_STRING);
	      }

	  }
       scanned_ports =  END_PORT_NUMBER - START_PORT_NUMBER;
    }

  fprintf (stdout, "\n");
  fprintf (stdout, "%s port scan on host %s completed.\n", PACKAGE, HOSTNAME_STRING);
  fprintf (stdout, "Total scanned ports: %d (%d - %d)\n", scanned_ports, START_PORT_NUMBER, END_PORT_NUMBER );
  fprintf (stdout, "Total open ports found: %d\n", open_ports);
  fprintf (stdout, "\n");

  quit (0);			/* exit the program */

  return 0;			/* avoid compilation warning */
}
